use crate::client::ONLINE_USERS;
use crate::events::{notify_all_clients_to_update_online_users, Event, CLIENT_SENDERS};
use crate::handles::handshake::HOST_ENABLED_ID_SET;
use crate::handles::redis::update_client_redis_data;
use crate::typing::message_typed::ClientMessageData;
use crate::utils::json_utils::get_current_timestamp;
use dashmap::DashMap;
use lazy_static::lazy_static;
use std::time::Duration;
use tokio::sync::mpsc;
use tokio::sync::mpsc::UnboundedSender;
use tokio::time::sleep;

lazy_static! {
    pub static ref refuse_procedure_map: DashMap<String, UnboundedSender<()>> = DashMap::new();
    pub static ref channel_hangup_procedure_map: DashMap<String, UnboundedSender<()>> =
        DashMap::new();
}

async fn send_inside_message(
    target_sender: &(String, u128),
    event_sender: &UnboundedSender<Event>,
    json_message: String,
    from_id: &String,
) {
    if let Err(e) = event_sender.send(Event::SendClientMessage(
        target_sender.clone(),
        json_message,
        false,
    )) {
        println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
    } else {
        println!("发送给用户id {} 独立消息成功", from_id);
    }
}

pub async fn handle_agora_call(
    client_message_data: &ClientMessageData,
    from_id: &String,
    event_sender: &UnboundedSender<Event>,
    connection_time: &u128,
) {
    let target_sender_which = (from_id.to_string(), connection_time.clone());
    // 对于非复杂性数据，不要使用serde_json，直接用format!宏即可
    match client_message_data.msg_type.as_str() {
        // 取消呼叫
        "CancelCall" => {
            let calling_to_id = &client_message_data.to_id;
            println!(
                "收到客户端取消呼叫 取消呼叫组: {} 呼叫方id {}",
                calling_to_id,
                from_id.to_string()
            );
            // 直接发送CmdHangup消息给当前用户
            let hangup_message_json = serde_json::json!({
                "msgType": "CmdHangup",
                "fromID": "0",
                "fromName": "Server",
                "toID": from_id,
                "msgData": {
                    "channelId": "",
                    "rtcToken": ""
                }
            })
            .to_string();
            if let Err(e) = event_sender.send(Event::SendClientMessage(
                target_sender_which.clone(),
                hangup_message_json,
                false,
            )) {
                println!("发送给用户id {} 独立消息失败：{:?}", from_id, e);
            }
            // 对同一个channelID的组，发送挂断消息
            if let Some(user_status) = ONLINE_USERS.get(from_id) {
                let mut the_current_user_data: Vec<&str> = user_status.split(',').collect();
                // 如果当前用户channelID为空，则中断后续操作，并且直接重新赋值即可
                if the_current_user_data[1].is_empty() {
                    the_current_user_data[1] = "";
                    the_current_user_data[0] = "idle";
                    the_current_user_data[6] = "0";
                    ONLINE_USERS.insert(
                        from_id.to_string(),
                        the_current_user_data.join(",").to_string(),
                    );
                    if let Err(e) = update_client_redis_data(
                        from_id,
                        the_current_user_data.join(",").to_string(),
                    )
                    .await
                    {
                        println!("更新redis数据失败：{:?}", e);
                    };
                    notify_all_clients_to_update_online_users().await;
                    return;
                }
                // 拆分toId为其他用户id，并且获取其数据信息
                let all_to_hangup_id_vec: Vec<&str> = calling_to_id.split(',').collect();
                for to_hangup_id in all_to_hangup_id_vec {
                    // 取消拒绝接听的线程队列
                    if let Some(ref_procedure_sender) =
                        refuse_procedure_map.get(&to_hangup_id.to_string())
                    {
                        if let Err(e) = ref_procedure_sender.send(()) {
                            println!("取消拒绝接听的线程队列失败：{:?}", e);
                        } else {
                            println!("取消拒绝接听的线程队列成功");
                        }
                    }
                    // 获取当前用户状态信息成功后，再进行额外操作
                    if let Some(the_other_caller_data) = ONLINE_USERS.get(to_hangup_id) {
                        let mut the_other_caller_data: Vec<&str> =
                            the_other_caller_data.split(',').collect();
                        // 如果处于同一频道，则发送挂断信息，
                        if the_other_caller_data[1] == the_current_user_data[1] {
                            let hangup_message_json = serde_json::json!({
                                "msgType": "CmdHangup",
                                "fromID": "0",
                                "fromName": "Server",
                                "toID": to_hangup_id,
                                "msgData": {
                                    "channelId": "",
                                    "rtcToken": ""
                                }
                            })
                            .to_string();
                            // 找到对应sender，发送CmdHangup消息
                            let target_sender = CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == to_hangup_id) // 使用 entry.key() 访问键
                                .map(|entry| entry.key().clone());
                            if let Some(target_sender) = target_sender {
                                if let Err(e) = event_sender.send(Event::SendClientMessage(
                                    target_sender,
                                    hangup_message_json,
                                    false,
                                )) {
                                    println!("发送给用户id {} 独立消息失败：{:?}", to_hangup_id, e);
                                    // 修改对应sender的数据
                                    the_other_caller_data[0] = "idle";
                                    the_other_caller_data[1] = "";
                                    the_other_caller_data[6] = "0";
                                    ONLINE_USERS.insert(
                                        to_hangup_id.to_string(),
                                        the_other_caller_data.join(",").to_string(),
                                    );
                                    if let Err(e) = update_client_redis_data(
                                        to_hangup_id,
                                        the_other_caller_data.join(",").to_string(),
                                    )
                                    .await
                                    {
                                        println!("更新redis数据失败：{:?}", e);
                                    };
                                }
                            } else {
                                println!("未找到对应的sender");
                            }
                        }
                    }
                }
                // 要求所有在线用户更新数据
                the_current_user_data[1] = "";
                the_current_user_data[0] = "idle";
                the_current_user_data[6] = "0";
                ONLINE_USERS.insert(
                    from_id.to_string(),
                    the_current_user_data.join(",").to_string(),
                );
                if let Err(e) =
                    update_client_redis_data(from_id, the_current_user_data.join(",").to_string())
                        .await
                {
                    println!("更新redis数据失败：{:?}", e);
                };
                notify_all_clients_to_update_online_users().await;
            }
        }
        // 对于通话类消息
        "Call" => {
            let calling_to_id = &client_message_data.to_id;
            println!(
                "收到客户端呼叫 呼叫方id {} 呼叫组id {}",
                from_id, calling_to_id
            );
            // 必须要指定呼叫对象
            if calling_to_id.is_empty() {
                let error_json = format!(
                    r#"{{"msgType": "Error", "fromID": "0", "fromName": "Server", "msgData": "请指定呼叫对象", "toID": "{}"}}"#,
                    &from_id
                );
                send_inside_message(&target_sender_which, &event_sender, error_json, &from_id)
                    .await;
                return;
            }
            // 不允许自己跟自己打电话
            if calling_to_id.eq(from_id) {
                let error_self_json = format!(
                    r#"{{"msgType": "Error", "fromID": "0", "fromName": "Server", "msgData": "不能给自己打电话", "toID": "{}"}}"#,
                    &from_id
                );
                send_inside_message(
                    &target_sender_which,
                    &event_sender,
                    error_self_json,
                    &from_id,
                )
                .await;
                return;
            }
            // 获取当前用户状态信息成功后，再进行额外操作
            if let Some(user_status) = ONLINE_USERS.get(from_id) {
                // 将获取的字符串用逗号拆分
                let mut the_user_data: Vec<&str> = user_status.split(',').collect();
                // 获取当前用户channelId
                let channel_id_now = if the_user_data[1].is_empty() {
                    format!("{}-{}", from_id, get_current_timestamp())
                } else {
                    the_user_data[1].to_string()
                };
                // 注意一下，这里的to_id有可能是多个，需要做一下过滤处理
                let calling_to_id_vec: Vec<&str> = calling_to_id.split(',').collect();
                // 储存需要计入CmdRefuse线程用户
                let mut refuse_thread_users: Vec<String> = vec![];
                for calling_to_id in calling_to_id_vec {
                    // 1.检查目标用户是否在线
                    if !ONLINE_USERS.contains_key(calling_to_id) {
                        let json_string = format!(
                            r#"{{"msgType":"Error","fromID":"0","fromName":"Server","msgData":"对方不在线","toID":"{}"}}"#,
                            &client_message_data.to_id
                        );
                        send_inside_message(
                            &target_sender_which,
                            &event_sender,
                            json_string,
                            &from_id,
                        )
                        .await;
                        continue;
                    }
                    // 2.检查目标用户是否为空闲状态
                    if let Some(user_status) = ONLINE_USERS.get(calling_to_id) {
                        // 将获取的字符串用逗号拆分
                        let mut the_another_user_data: Vec<&str> = user_status.split(',').collect();
                        // 如果不为idle，不允许后续操作
                        if the_another_user_data[0] != "idle" {
                            let json_string = format!(
                                r#"{{"msgType":"Error","fromID":"0","fromName":"Server","msgData":"对方正在呼叫中","toID":"{}"}}"#,
                                from_id
                            );
                            send_inside_message(
                                &target_sender_which,
                                &event_sender,
                                json_string,
                                &from_id,
                            )
                            .await;
                            continue;
                        }
                        // 3.如果为空闲状态，准备发送Call类型消息
                        let call_message_json = serde_json::json!({
                            "msgType": "Call",
                            "fromID": from_id,
                            "fromName": client_message_data.from_name,
                            "toID": calling_to_id,
                            "msgData": {
                                "channelId": channel_id_now,
                                "rtcToken": ""
                            }
                        })
                        .to_string();
                        // 通知被呼叫方的sender发送数据
                        let target_sender = CLIENT_SENDERS
                            .iter()
                            .find(|entry| entry.key().0 == calling_to_id) // 使用 entry.key() 访问键
                            .map(|entry| entry.key().clone()); // 使用 entry.value() 访问值
                        if let Some(sender) = target_sender {
                            if let Err(e) = event_sender.send(Event::SendClientMessage(
                                sender.clone(),
                                call_message_json,
                                false,
                            )) {
                                println!("发送给用户id {:?} 独立消息失败：{:?}", sender, e);
                                continue;
                            }
                        } else {
                            println!("找不到对应的sender，无法发送客户端消息！");
                            continue;
                        }
                        println!("发送给用户id {} 独立消息成功", from_id);
                        // 修改对应用户的状态数据
                        the_another_user_data[0] = "callin";
                        the_another_user_data[1] = channel_id_now.as_str();
                        the_another_user_data[6] = "0";
                        let the_another_user_data_joined = the_another_user_data.join(",");
                        // 更新状态数据
                        ONLINE_USERS.insert(
                            calling_to_id.to_string(),
                            the_another_user_data_joined.clone(),
                        );
                        // 修改redis数据
                        if let Err(e) = update_client_redis_data(
                            calling_to_id,
                            the_another_user_data_joined.clone(),
                        )
                        .await
                        {
                            println!("更新redis数据失败：{:?} 用户id {}", e, calling_to_id);
                        } else {
                            println!("更新redis数据成功");
                        }
                        refuse_thread_users.push(calling_to_id.to_string());
                    }
                }
                // 如果成功发送了呼叫信息，则根据当前用户状态发送消息通知
                if refuse_thread_users.len() > 0 {
                    // 如果当前用户本身为idle，则发送CmdCall
                    if the_user_data[0] == "idle" || the_user_data[0] == "calling" {
                        let cmd_call_message_json = serde_json::json!({
                            "msgType": "CmdCall",
                            "fromID": calling_to_id,
                            "fromName": client_message_data.from_name,
                            "toID": from_id,
                            "msgData": {
                                "channelId": channel_id_now,
                                "rtcToken": ""
                            }
                        })
                        .to_string();
                        // 更新状态数据
                        the_user_data[0] = "calling";
                        the_user_data[1] = channel_id_now.as_str();
                        the_user_data[6] = "0";
                        // 在连接成功之后再设置为主持人，在此之前一律将所有用户的isHost数据段设置为0
                        let the_user_data_joined = the_user_data.join(",");
                        ONLINE_USERS.insert(from_id.to_string(), the_user_data_joined.clone());
                        // 修改redis数据
                        if let Err(e) =
                            update_client_redis_data(from_id, the_user_data_joined).await
                        {
                            println!("更新redis数据失败：{:?} 用户id {}", e, from_id);
                        } else {
                            println!("更新redis数据成功");
                        }
                        send_inside_message(
                            &target_sender_which,
                            event_sender,
                            cmd_call_message_json,
                            &from_id,
                        )
                        .await;
                        // 最后广播用户更新
                        notify_all_clients_to_update_online_users().await;
                        // 开启定时器线程任务，如果所有用户20s内没有接听，则挂断，发送所有CmdRefuse数据并重置
                        for user_id in refuse_thread_users {
                            if let Some(current_user_status_data) = ONLINE_USERS.get(&user_id) {
                                // 设置线程任务
                                let user_id_clone = user_id.clone();
                                let from_id = from_id.clone();
                                let event_sender = event_sender.clone();
                                let (cancel_tx, mut cancel_rx) = mpsc::unbounded_channel::<()>();
                                refuse_procedure_map.insert(user_id, cancel_tx);
                                tokio::spawn(async move {
                                    tokio::select! {
                                        _ = sleep(Duration::from_secs(20)) => {
                                            // 20秒内没有接听，准备挂断通话
                                            let mut current_user_data_vec: Vec<&str> = current_user_status_data.split(',').collect();
                                            // 提前取出channelID，判断是否还有人在频道里面
                                            let current_channel_id = current_user_data_vec[1];
                                            current_user_data_vec[0] = "idle";
                                            current_user_data_vec[1] = "";
                                            current_user_data_vec[6] = "0";
                                            // 更新本地数据
                                            ONLINE_USERS.insert(
                                                user_id_clone.clone(),
                                                current_user_data_vec.join(","),
                                            );
                                            // 更新redis数据
                                            if let Err(e) = update_client_redis_data(
                                                user_id_clone.as_str(),
                                                current_user_data_vec.join(","),
                                            )
                                            .await {
                                                println!("更新redis数据失败：{:?} 用户id {}", e, user_id_clone);
                                            } else {
                                                println!("通知挂断refuse线程，更新redis数据成功");
                                            }
                                            // 对其发送CmdHangup，对发起方发送CmdRefuse
                                            // 发起方的isHost一定是1，找到该用户即可
                                            let host_user_id_now = ONLINE_USERS
                                                .iter()
                                                .find(|entry| {
                                                    entry.value().split(',').collect::<Vec<_>>()[1]
                                                        == current_channel_id
                                                        && entry.value().split(',').collect::<Vec<_>>()[6]
                                                            == "1"
                                                })
                                                .map(|entry| entry.key().clone());
                                            // 再找到对应的Sender
                                            let host_user_sender_which = if let Some(host_user_id) = host_user_id_now {
                                                CLIENT_SENDERS
                                                    .iter()
                                                    .find(|entry| entry.key().0 == host_user_id)
                                                    .map(|entry| entry.key().clone())
                                            } else {
                                                None
                                            };
                                            if let Some(host_user_sender_which) = host_user_sender_which {
                                                // 对其发送消息
                                                send_inside_message(
                                                    &host_user_sender_which,
                                                    &event_sender,
                                                    serde_json::json!({
                                                        "msgType": "CmdRefuse",
                                                        "fromID": user_id_clone,
                                                        "fromName": "Unknown",
                                                        "toID": &host_user_sender_which.0,
                                                        "msgData": {
                                                            "channelId": "",
                                                            "rtcToken": ""
                                                        }
                                                    }).to_string(),
                                                    &host_user_sender_which.0,
                                                )
                                                .await;
                                            }
                                            // 向当前用户Sender发送CmdHangup要求其挂断
                                            let hangup_user_sender_which = if let Some(hangup_user_sender_which) =
                                            CLIENT_SENDERS.iter().find(|entry| entry.key().0 == user_id_clone) {
                                                Some(hangup_user_sender_which.key().clone())
                                            } else {
                                                None
                                            };
                                            if let Some(hangup_user_sender_which) = hangup_user_sender_which {
                                                // 对其发送消息
                                                send_inside_message(
                                                    &hangup_user_sender_which,
                                                    &event_sender,
                                                    serde_json::json!({
                                                        "msgType": "CmdHangup",
                                                        "fromID": "0",
                                                        "fromName": "Server",
                                                        "toID": from_id,
                                                        "msgData": {
                                                            "channelId": "",
                                                            "rtcToken": ""
                                                        }
                                                    }).to_string(),
                                                    &user_id_clone.to_string(),
                                                ).await;
                                            }
                                            // 检查剩余频道人员数量
                                            let left_users: Vec<(String, String)> = ONLINE_USERS.iter()
                                                .filter(|entry| entry.value().split(',').collect::<Vec<_>>()[1] == current_channel_id)
                                                .map(|entry| (entry.key().clone(), entry.value().clone())) // 克隆键和值
                                                .collect();
                                            if left_users.len() <= 1 {
                                                let only_left_user = left_users.get(0);
                                                if let Some((user_id, user_data)) = only_left_user {
                                                    let mut user_data = user_data.split(',').collect::<Vec<&str>>();
                                                    {
                                                        let user_is_host = user_data[1] == "1";
                                                        if user_is_host {
                                                            // 发起方
                                                            // 找到发起方的Sender
                                                            if let Some(host_user_sender_which) =
                                                            CLIENT_SENDERS.iter().find(|entry| entry.key().0 == user_id.to_string()) {
                                                                // 更新状态数据到OnlineUsers
                                                                user_data[0] = "idle";
                                                                user_data[1] = "";
                                                                user_data[6] = "0";
                                                                ONLINE_USERS.insert(user_id.to_string(), user_data.join(","));
                                                                // 更新redis数据
                                                                if let Err(e) = update_client_redis_data(&user_id.to_string(), user_data.join(",")).await {
                                                                    println!("只有一人在会议中，更新redis数据失败: {}", e)
                                                                } else {
                                                                    println!("只有一人在会议中，更新redis数据成功")
                                                                }
                                                                // 对其发送CmdCancelCall
                                                                send_inside_message(
                                                                    host_user_sender_which.key(),
                                                                    &event_sender,
                                                                    serde_json::json!({
                                                                        "msgType": "CmdCancelCall",
                                                                        "fromID": "0",
                                                                        "fromName": "Server",
                                                                        "toID": host_user_sender_which.key().0,
                                                                       "msgData": {}
                                                                    }).to_string(), user_id,
                                                                ).await;
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                            // 要求所有用户更新在线人员列表
                                            notify_all_clients_to_update_online_users().await;
                                        }
                                        _ = cancel_rx.recv() => {
                                            // 线程被取消
                                            println!("用户已接听或者主动挂断 {}", user_id_clone);
                                        }
                                    }
                                });
                            }
                        }
                    }
                }
            }
        }
        // 拒接电话
        "Refuse" => {
            // 当前用户拒接电话之后，直接给当前用户发送CmdHangup消息
            // 主动拒绝之后，也需要取消refuse_procedure_map的线程任务
            if let Some(ref_procedure_sender) = refuse_procedure_map.get(from_id) {
                if let Err(e) = ref_procedure_sender.send(()) {
                    println!("取消拒绝接听的线程队列失败：{:?}", e);
                } else {
                    println!("取消拒绝接听的线程队列成功");
                }
            }
            // 给当前from_id用户发送CmdHangup，表示已经成功拒接了电话，并且修改了状态
            if let Some(current_user_data) = ONLINE_USERS.get(from_id) {
                let mut current_user_data_vec: Vec<&str> = current_user_data.split(',').collect();
                let hangup_refuse_message = serde_json::json!({
                    "msgType": "CmdHangup",
                    "fromID": "0",
                    "fromName": "Server",
                    "toID": from_id,
                    "msgData": {
                        "channelId": "",
                        "rtcToken": ""
                    }
                })
                .to_string();
                // 修改状态数据
                current_user_data_vec[0] = "idle";
                current_user_data_vec[1] = "";
                current_user_data_vec[6] = "0";
                let current_user_data_joined = current_user_data_vec.join(",");
                ONLINE_USERS.insert(from_id.to_string(), current_user_data_joined.clone());
                // 修改redis数据
                if let Err(e) = update_client_redis_data(from_id, current_user_data_joined).await {
                    println!("更新redis数据失败：{:?} 用户id {}", e, from_id);
                } else {
                    println!("更新redis数据成功");
                }
                // 直接给sender发送数据通知
                send_inside_message(
                    &target_sender_which,
                    event_sender,
                    hangup_refuse_message,
                    &from_id,
                )
                .await;
            }
            // 注意，拒接电话只能有一个toID
            // 在这里，toID的状态数据由客户端维护
            // 找到对应toId的sender
            if let Some(target_sender) = CLIENT_SENDERS
                .iter()
                .find(|entry| entry.key().0 == client_message_data.to_id.to_string())
                .map(|entry| entry.key().clone())
            {
                // 直接发送CmdHangup数据
                let cmd_refuse_message = serde_json::json!({
                    "msgType": "CmdRefuse",
                    "fromID": client_message_data.to_id,
                    "fromName": "Server",
                    "toID": from_id,
                    "msgData": {
                        "channelId": "",
                        "rtcToken": ""
                    }
                })
                .to_string();
                send_inside_message(
                    &target_sender,
                    event_sender,
                    cmd_refuse_message,
                    &client_message_data.to_id,
                )
                .await;
            } else {
                println!("找不到toID对应的sender");
            }
            // 更新所有用户状态数据
            notify_all_clients_to_update_online_users().await;
        }
        // 主持人结束通话
        "EndMeeting" => {
            // 要求判断是否为主持人，只有主持人可以结束通话
            if let Some(current_user_data) = ONLINE_USERS.get(from_id) {
                let current_user_data_vec: Vec<&str> = current_user_data.split(',').collect();
                // 如果不是主持人，不允许发送该消息
                if current_user_data_vec[6] != "1" {
                    // 发送错误信息
                    let error_message = serde_json::json!({
                        "msgType": "Error",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": from_id,
                        "msgData": "只有主持人可以结束会议！"
                    })
                    .to_string();
                    send_inside_message(
                        &target_sender_which,
                        event_sender,
                        error_message,
                        &from_id,
                    )
                    .await;
                } else {
                    // 如果是主持人，则向所有人发送CmdEndMeeting消息，并且清理所有数据
                    // 提取出目前channel_id
                    let channel_id = current_user_data_vec[1];
                    if channel_id != "" {
                        // 从ONLINE_USERS中过滤筛选出所有与channelID相同的用户id集合
                        let users_to_notify = ONLINE_USERS
                            .iter()
                            .filter(|entry| {
                                entry.value().split(",").collect::<Vec<_>>()[2] == channel_id
                            })
                            .map(|entry| entry.key().to_string())
                            .collect::<Vec<String>>();
                        // 再根据上面的id，找到所有sender，并且修改所有对应ONLINE_USERS数据
                        for user_id_current_chat in users_to_notify.iter() {
                            if let Some(current_user_data) = ONLINE_USERS.get(user_id_current_chat)
                            {
                                let mut current_user_data_vec: Vec<&str> =
                                    current_user_data.split(',').collect();
                                current_user_data_vec[0] = "idle";
                                current_user_data_vec[1] = "";
                                current_user_data_vec[6] = "0";
                                let current_user_data_joined = current_user_data_vec.join(",");
                                ONLINE_USERS.insert(
                                    user_id_current_chat.to_string(),
                                    current_user_data_joined.clone(),
                                );
                                // 更新对应redis数据
                                if let Err(e) = update_client_redis_data(
                                    user_id_current_chat.as_str(),
                                    current_user_data_joined,
                                )
                                .await
                                {
                                    println!(
                                        "更新redis数据失败：{:?} 用户id {}",
                                        e, user_id_current_chat
                                    );
                                } else {
                                    println!("更新redis数据成功");
                                }
                            }
                        }
                        for user_id in users_to_notify {
                            if let Some(target_sender) = CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == user_id)
                                .map(|entry| entry.key().clone())
                            {
                                // 找到这个sender标记符之后，发送消息CmdEndMeeting
                                send_inside_message(
                                    &target_sender,
                                    event_sender,
                                    serde_json::json!({
                                        "msgType": "CmdEndMeeting",
                                        "fromID": "0",
                                        "fromName": "Server",
                                        "toID": user_id,
                                        "msgData": {
                                            "channelId": "",
                                            "rtcToken": ""
                                        }
                                    })
                                    .to_string(),
                                    &user_id,
                                )
                                .await;
                            }
                        }
                        // 发送完毕之后，要求更新所有用户数据
                        notify_all_clients_to_update_online_users().await;
                    }
                }
            }
        }
        // 通话过程中挂断
        "Hangup" => {
            // 挂断通话不是拒接，不需要处理refuse_procedure_map中的线程消息
            if let Some(current_user_data) = ONLINE_USERS.get(from_id) {
                // 直接修改对应数据即可
                let mut current_user_data_vec: Vec<&str> = current_user_data.split(',').collect();
                let is_current_user_host = current_user_data_vec[6] == "1";
                let current_chatting_channel_id = current_user_data_vec[1];
                current_user_data_vec[0] = "idle";
                current_user_data_vec[1] = "";
                // 无论是否是主持人，都需要发送对应消息，更新数据以及同步redis
                current_user_data_vec[6] = "0";
                let current_user_data_joined = current_user_data_vec.join(",");
                ONLINE_USERS.insert(from_id.to_string(), current_user_data_joined.clone());
                // 更新redis数据
                if let Err(e) = update_client_redis_data(from_id, current_user_data_joined).await {
                    println!("更新redis数据失败：{:?} 用户id {}", e, from_id);
                } else {
                    println!("更新redis数据成功");
                }
                // 直接向其发送CmdHangup即可
                send_inside_message(
                    &target_sender_which,
                    event_sender,
                    serde_json::json!({
                        "msgType": "CmdHangup",
                        "fromID": "0",
                        "fromName": "Server",
                        "toID": from_id,
                        "msgData": {
                            "channelId": "",
                            "rtcToken": ""
                        }
                    })
                    .to_string(),
                    &from_id,
                )
                .await;
                // 如果当前用户是主持人，主持人退出之后，需要在剩下的用户中决定新主持人
                if is_current_user_host {
                    // 过滤出剩下来的channelID同一的在线用户，并且整合为Vec集合
                    let remaining_users: Vec<String> = ONLINE_USERS
                        .iter()
                        .filter(|entry| {
                            let v_vec: Vec<&str> = entry.value().split(',').collect();
                            v_vec[1] == current_chatting_channel_id
                        })
                        .map(|entry| entry.key().to_string())
                        .collect();
                    // 没有人参与会议了，直接退出即可
                    if remaining_users.len() == 0 {
                        println!("当前频道没有人员，请重新发起通话");
                    }
                    // 如果余留下的用户数量仅为1，则开启线程，20s内如果该频道与会人数仍为1，则对剩下来的用户发送CmdHangup指令
                    // 如果大于2人，则需要判断主持人
                    else {
                        // 无论如何，都要向他们发送CmdLeave消息
                        for user_id in remaining_users.iter() {
                            if let Some(target_sender) = CLIENT_SENDERS
                                .iter()
                                .find(|entry| entry.key().0 == *user_id)
                                .map(|entry| entry.key().clone())
                            {
                                // 找到这个sender标记符之后，发送消息CmdEndMeeting
                                let cmd_leave_message = serde_json::json!({
                                    "msgType": "CmdLeave",
                                    "fromID": &from_id,
                                    "fromName": "Unknown",
                                    "toID": user_id,
                                    "msgData": {}
                                })
                                .to_string();
                                send_inside_message(
                                    &target_sender,
                                    event_sender,
                                    cmd_leave_message,
                                    user_id,
                                )
                                .await;
                            }
                        }
                        if remaining_users.len() == 1 {
                            let (cancel_tx, mut cancel_rx) = mpsc::unbounded_channel::<()>();
                            channel_hangup_procedure_map
                                .insert(current_chatting_channel_id.to_string(), cancel_tx);
                            let channel_id_clone = current_chatting_channel_id.to_string();
                            let ev_clone = event_sender.clone();
                            // 开启一个tokio线程，使用async move
                            tokio::spawn(async move {
                                tokio::select! {
                                    _ = tokio::time::sleep(Duration::from_secs(20)) => {
                                        // 20s内如果该频道与会人数仍为1，则对剩下来的用户发送CmdHangup指令
                                        println!("20s内如果该频道与会人数仍为1，则对剩下来的用户发送CmdHangup指令 {:?}", remaining_users);
                                        if let Some(remaining_user_id) = remaining_users.get(0) {
                                             {
                                                if let Some(remain_user_info_data) = ONLINE_USERS.get(remaining_user_id){
                                                    // 找到对应Sender
                                                    if let Some(target_sender_which) = CLIENT_SENDERS.iter()
                                                        .find(|entry| entry.key().0 == *remaining_user_id) {
                                                        // 修改状态数据
                                                        let mut user_info_data_for_remained = remain_user_info_data.split(',').collect::<Vec<_>>();
                                                        user_info_data_for_remained[0] = "idle";
                                                        user_info_data_for_remained[1] = "";
                                                        user_info_data_for_remained[6] = "0";
                                                        let user_info_data_for_remained_joined = user_info_data_for_remained.join(",");
                                                        ONLINE_USERS.insert(remaining_user_id.to_string(), user_info_data_for_remained_joined.clone());
                                                        if let Err(e) = update_client_redis_data(remaining_user_id, user_info_data_for_remained_joined).await {
                                                            println!("更新redis数据失败：{:?} 用户id {}", e, remaining_user_id);
                                                        } else {
                                                            println!("更新redis数据成功");
                                                        }
                                                        // 发送CmdHangup
                                                        send_inside_message(
                                                            &target_sender_which.key(),
                                                            &ev_clone,
                                                            serde_json::json!({
                                                                "msgType": "CmdHangup",
                                                                "fromID": "0",
                                                                "fromName": "Server",
                                                                "toID": remaining_user_id,
                                                                "msgData": {
                                                                    "channelId":"",
                                                                }
                                                            }).to_string(),
                                                            &remaining_user_id,
                                                        ).await;
                                                        // 要求所有用户更新数据
                                                        notify_all_clients_to_update_online_users().await;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    _ = cancel_rx.recv() => {
                                        // 如果收到取消信号，则直接结束线程
                                        println!("收到取消信号，有新的人员加入到会议，会议频道号 {} 结束线程", channel_id_clone);
                                    }
                                }
                            });
                        }
                        // 如果余下用户数量超过2人及以上，则需要判断应该让谁让主持人
                        else {
                            // 分组：1.筛选出id在HOST_ENABLED_ID_SET中的用户，即允许成为主持人的用户，2.筛选出id不在HOST_ENABLED_ID_SET中的用户，即不允许成为主持人的用户
                            // 注意，这里只需要修改一下状态数据即可，不用单独发送消息
                            let (allowed_users, disallowed_users): (Vec<_>, Vec<_>) =
                                remaining_users
                                    .iter()
                                    .partition(|user_id| HOST_ENABLED_ID_SET.contains(user_id.as_str()));
                            if !allowed_users.is_empty() {
                                // 让第一位id的用户成为主持人
                                if let Some(allowed_user_id) = allowed_users.get(0) {
                                    if let Some(allowed_user_info_data) =
                                        ONLINE_USERS.get(allowed_user_id.as_str())
                                    {
                                        {
                                            // 修改状态数据
                                            let mut user_info_data_for_allowed =
                                                allowed_user_info_data
                                                    .split(',')
                                                    .collect::<Vec<_>>();
                                            user_info_data_for_allowed[6] = "1";
                                            // 修改online数据信息
                                            let user_info_data_for_allowed_joined =
                                                user_info_data_for_allowed.join(",");
                                            ONLINE_USERS.insert(
                                                allowed_user_id.to_string(),
                                                user_info_data_for_allowed_joined.clone(),
                                            );
                                            if let Err(e) = update_client_redis_data(
                                                allowed_user_id,
                                                user_info_data_for_allowed_joined.clone(),
                                            )
                                            .await
                                            {
                                                println!(
                                                    "更新redis数据失败：{:?} 用户id {}",
                                                    e, allowed_user_id
                                                )
                                            } else {
                                                println!("更新redis数据成功");
                                            }
                                        }
                                    }
                                }
                            } else if !disallowed_users.is_empty() {
                                // 让第一位
                                if let Some(disallowed_user_id) = disallowed_users.get(0) {
                                    // 完全照搬上面的逻辑
                                    if let Some(disallowed_user_info_data) =
                                        ONLINE_USERS.get(disallowed_user_id.as_str())
                                    {
                                        {
                                            let mut current_host_data = disallowed_user_info_data
                                                .split(',')
                                                .collect::<Vec<_>>();
                                            current_host_data[6] = "1";
                                            let current_host_data_joined =
                                                current_host_data.join(",");
                                            // 更新ONLINE_USERS数据
                                            ONLINE_USERS.insert(
                                                disallowed_user_id.to_string(),
                                                current_host_data_joined.clone(),
                                            );
                                            if let Err(e) = update_client_redis_data(
                                                disallowed_user_id,
                                                current_host_data_joined.clone(),
                                            )
                                            .await
                                            {
                                                println!(
                                                    "更新redis数据失败：{:?} 用户id {}",
                                                    e, disallowed_user_id
                                                )
                                            } else {
                                                println!("更新redis数据成功");
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                // 无论如何都要发送通知在线人员更新
                notify_all_clients_to_update_online_users().await;
            }
        }
        // 接听通话，这个是目前最重要的逻辑部分
        // 在这里要处理如下事项
        // 1.取消20s内挂断某个用户的线程；
        // 2.若加入的频道号之前只有一个用户，那么目前频道是超过2人的，那么要取消20s内退出的线程；
        // 3.修改状态并且广播出去
        // 4.决定主持人。一般来说，谁发起了这个频道的人，并且具备成为主持人的资格，那么就直接成为主持人，否则就让第一个加入的人成为主持人。
        "Connect" => {
            println!("收到客户端Connect消息连接 {} 频道号 {}", &from_id, client_message_data.msg_data.get("channelID").unwrap_or(&serde_json::Value::String("null".to_string())));
            if let Some(channel_id) = client_message_data.msg_data.get("channelID") {
                // 这里的from_id就是被呼叫方，to_id就是呼叫方
                // 首先先取消掉所有的挂断from_id的任务
                if let Some(hangup_personnel) = refuse_procedure_map.get(from_id) {
                    if let Err(e) = hangup_personnel.send(()) {
                        println!("挂断personnel的任务终结失败 {:?}", e)
                    } else {
                        println!("挂断personnel的任务已经发送成功")
                    }
                }
                // 将channel_id转换为字符串，并且进行解析
                if let Some(channel_id_str) = channel_id.as_str() {
                    // 检查一下是否存在挂断channel的任务，如果存在，对齐发送消息，然后关闭
                    if let Some(hangup_channel_task) = channel_hangup_procedure_map.get(channel_id_str) {
                        if let Err(e) = hangup_channel_task.send(()) {
                            println!("挂断channel的任务终结失败 {:?}", e)
                        } else {
                            println!("挂断channel的任务已经发送成功")
                        }
                    }
                    // 如果当前channel已经有主持人，则不做处理
                    let already_had_host = ONLINE_USERS.iter()
                        .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] ==channel_id_str)
                        .any(|entry| entry.value().split(",").collect::<Vec<_>>()[6] == "1");
                    if already_had_host {
                        println!("当前channel已经有主持人，直接返回 {}", channel_id_str)
                    } else {
                        println!("当前channel没有主持人，开始处理 {}", channel_id_str);
                        // 首先判断发起方，也就是channel_id用_分割之后的第一个元素，是否具备成为主持人的资格
                        if let Some(channel_id_first_element) = channel_id_str.split("_").collect::<Vec<_>>().get(0) {
                            if HOST_ENABLED_ID_SET.contains(&channel_id_first_element.to_string()) {
                                // 具备成为主持人资格，直接修改其对应数据即可
                                if let Some(host_data) = ONLINE_USERS.get(&channel_id_first_element.to_string()) {
                                    let mut host_data_vec = host_data.split(",").collect::<Vec<_>>();
                                    host_data_vec[6] = "1";
                                    let host_data_joined = host_data_vec.join(",");
                                    // 直接再次更新online_users
                                    ONLINE_USERS.insert(
                                        channel_id_first_element.to_string(),
                                        host_data_joined.clone(),
                                    );
                                    if let Err(e) = update_client_redis_data(
                                        channel_id_first_element,
                                        host_data_joined.clone(),
                                    )
                                    .await
                                    {
                                        println!(
                                            "更新redis数据失败：{:?} 用户id {} 成为主持人",
                                            e, channel_id_first_element
                                        )
                                    } else {
                                        println!("更新redis数据成功");
                                    }
                                }
                            } else {
                                // 获取所有同channel_id的用户，在ONLINE_USERS中寻找第一个满足条件的id
                                let all_in_group_chatters_id: Vec<String> = ONLINE_USERS.iter()
                                    .filter(|entry| entry.value().split(",").collect::<Vec<_>>()[1] ==channel_id_str)
                                    .map(|entry| entry.key().clone())
                                    .collect::<Vec<String>>();
                                // 记录是否找到主持人对应id
                                let mut is_host_found = false;
                                // 在all_in_group_chatters_id中，找寻第一个在HOST_ENABLED_ID_SET中的id
                                for id in all_in_group_chatters_id {
                                    if HOST_ENABLED_ID_SET.contains(&id) {
                                        // 具备成为主持人资格，直接修改其对应数据即可
                                        if let Some(host_data) = ONLINE_USERS.get(&id) {
                                            let mut host_data_vec = host_data.split(",").collect::<Vec<_>>();
                                            host_data_vec[6] = "1";
                                            is_host_found = true;
                                            // 更新对应数据
                                            let host_data_joined = host_data_vec.join(",");
                                            ONLINE_USERS.insert(
                                                id.to_string(),
                                                host_data_joined.clone(),
                                            );
                                            if let Err(e) = update_client_redis_data(
                                                &id,
                                                host_data_joined.clone(),
                                            )
                                            .await
                                            {
                                                println!(
                                                    "更新redis数据失败：{:?} 用户id {} 成为主持人",
                                                    e, id
                                                )
                                            } else {
                                                println!("更新redis数据成功");
                                            }
                                            break;
                                        }
                                    }
                                }
                                if !is_host_found {
                                    println!("当前channel没有主持人，并且没有找到合适的主持人，直接派遣第一个id即可 {}", channel_id_str);
                                }
                            }
                        }
                    }
                }
                // 最后广播所有用户更新状态
                notify_all_clients_to_update_online_users().await;
            } else {
                println!("客户端Connect消息 缺乏channel-id，拒绝处理消息")
            }
        }
        // 处理静音
        "Mute" => {

        }
        _ => {}
    }
}
